namespace MSProjectAddIn
{
   using System;
   using System.Collections;
   using System.Reflection;
   using Microsoft.Office.Core;
   using Extensibility;
   using System.Runtime.InteropServices;

   #region Read me for Add-in installation and setup information.
   // When run, the Add-in wizard prepared the registry for the Add-in.
   // At a later time, if the Add-in becomes unavailable for reasons such as:
   //    1) You moved this project to a computer other than which is was originally created on.
   //    2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
   //    3) Registry corruption.
   // you will need to re-register the Add-in by building the MyAddin21Setup project 
   // by right clicking the project in the Solution Explorer, then choosing install.
   #endregion

   /// <summary>
   ///   The object for implementing an Add-in.
   /// </summary>
   /// <seealso class='IDTExtensibility2' />
   [GuidAttribute("9BC229A5-48A3-45A2-BA55-E0828B1C353A"), ProgId("MSProjectAddIn.Connect")]
   public class Connect : Object, Extensibility.IDTExtensibility2
   {
      /// <summary>
      ///      Implements the constructor for the Add-in object.
      ///      Place your initialization code within this method.
      /// </summary>

      private String m_strTTPDatabase;
      private String m_strTTPUsername;
      private String m_strTTPPassword;
      private CommandBarButton cbbuttonProject;
      private object applicationObject;
      private object addInInstance;

      public Connect()
      {
      }

      /// <summary>
      ///      Implements the OnConnection method of the IDTExtensibility2 interface.
      ///      Receives notification that the Add-in is being loaded.
      /// </summary>
      /// <param term='application'>
      ///      Root object of the host application.
      /// </param>
      /// <param term='connectMode'>
      ///      Describes how the Add-in is being loaded.
      /// </param>
      /// <param term='addInInst'>
      ///      Object representing this Add-in.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom)
      {
         applicationObject = application;
         addInInstance = addInInst;
      }

      /// <summary>
      ///      Implements the OnDisconnection method of the IDTExtensibility2 interface.
      ///      Receives notification that the Add-in is being unloaded.
      /// </summary>
      /// <param term='disconnectMode'>
      ///      Describes how the Add-in is being unloaded.
      /// </param>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)
      {
      }

      /// <summary>
      ///      Implements the OnAddInsUpdate method of the IDTExtensibility2 interface.
      ///      Receives notification that the collection of Add-ins has changed.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnAddInsUpdate(ref System.Array custom)
      {
      }

      /// <summary>
      ///      Implements the OnStartupComplete method of the IDTExtensibility2 interface.
      ///      Receives notification that the host application has completed loading.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnStartupComplete(ref System.Array custom)
      {
         CommandBars oCommandBars;
         CommandBar oStandardBar;

         try
         {
            oCommandBars = (CommandBars)applicationObject.GetType().InvokeMember("CommandBars", BindingFlags.GetProperty , null, applicationObject ,null);
         }
         catch (Exception)
         {
            // Outlook has the CommandBars collection on the Explorer object.
            object oActiveExplorer;
            oActiveExplorer = applicationObject.GetType().InvokeMember("ActiveExplorer",BindingFlags.GetProperty,null,applicationObject,null);
            oCommandBars = (CommandBars)oActiveExplorer.GetType().InvokeMember("CommandBars",BindingFlags.GetProperty,null,oActiveExplorer,null);
         }

         // Set up a custom button on the "Standard" commandbar.
         try
         {
            oStandardBar = oCommandBars["Project"];
         }
         catch (Exception)
         {
            // Access names its main toolbar Database.
            oStandardBar = oCommandBars["TestTrack Pro Defects"];      
         }

         // In case the button was not deleted, use the exiting one.
         try
         {
            cbbuttonProject = (CommandBarButton)oStandardBar.Controls["TestTrack Pro Defects"];
         }
         catch (Exception)
         {
            object omissing = System.Reflection.Missing.Value;
            cbbuttonProject = (CommandBarButton) oStandardBar.Controls.Add(1, omissing , omissing , omissing , omissing);
            cbbuttonProject.Caption = "TestTrack Pro Defects";
            cbbuttonProject.Style = MsoButtonStyle.msoButtonCaption;
         }

         // The following items are optional, but recommended. 
         // The Tag property lets you quickly find the control 
         // and helps MSO keep track of it when more than
         // one application window is visible. The property is required
         // by some Office applications and should be provided.
         cbbuttonProject.Tag = "TestTrack Pro Defects";

         // The OnAction property is optional but recommended. 
         // It should be set to the ProgID of the add-in, so that if
         // the add-in is not loaded when a user presses the button,
         // MSO loads the add-in automatically and then raises
         // the Click event for the add-in to handle. 
         cbbuttonProject.OnAction = "!<MyCOMAddin.Connect>";

         cbbuttonProject.Visible = true;
         cbbuttonProject.Click += new Microsoft.Office.Core._CommandBarButtonEvents_ClickEventHandler(this.cbbuttonProject_Click);

         object oName = applicationObject.GetType().InvokeMember("Name",BindingFlags.GetProperty,null,applicationObject,null);

         // Display a simple message to show which application you started in.
         // System.Windows.Forms.MessageBox.Show("This Addin is loaded by " + oName.ToString()   , "MyCOMAddin");
         oStandardBar = null;
         oCommandBars = null;

         m_strTTPDatabase = "";
         m_strTTPUsername = "";
         m_strTTPPassword = "";
      }

      /// <summary>
      ///      Implements the OnBeginShutdown method of the IDTExtensibility2 interface.
      ///      Receives notification that the host application is being unloaded.
      /// </summary>
      /// <param term='custom'>
      ///      Array of parameters that are host application specific.
      /// </param>
      /// <seealso class='IDTExtensibility2' />
      public void OnBeginShutdown(ref System.Array custom)
      {
      }

      private void SomethingChanged(msprjlib.Task tsk, msprjlib.PjField Field, object newVal, ref bool Info) 
      {
         Seapine.ttsoapcgi cgiEngine = new Seapine.ttsoapcgi();
         long lSessionID = cgiEngine.DatabaseLogon( m_strTTPDatabase, m_strTTPUsername, m_strTTPPassword);

         Seapine.CDefect defect;
         if ( tsk.EnterpriseNumber1 > 0)
         {
            defect = cgiEngine.getDefect(lSessionID, Convert.ToInt32(tsk.EnterpriseNumber1), "test", false);

            switch (Field)
            {
               case msprjlib.PjField.pjTaskFinish:
//						defect.CField.SetValue( tsk.ActualFinish, 0 );
                  break;
               default:
                  break;
            }

            cgiEngine.saveDefect(lSessionID, defect);
         }

         cgiEngine.DatabaseLogoff(lSessionID);
      }

      private void cbbuttonProject_Click(CommandBarButton cmdBarbutton, ref bool cancel)
      {
         long lSessionID = -1;
         Seapine.ttsoapcgi cgiEngine = new Seapine.ttsoapcgi();

         try
         {
            Seapine.CDatabase[] adb = cgiEngine.getDatabaseList();

            Login login = new Login(adb);
            login.ShowDialog();
            if (login.m_strDatabase.Length > 0)
            {
               m_strTTPDatabase = login.m_strDatabase;
               m_strTTPUsername = login.m_strUsername;
               m_strTTPPassword = login.m_strPassword;
               try
               {
                  lSessionID = cgiEngine.DatabaseLogon( m_strTTPDatabase, m_strTTPUsername, m_strTTPPassword);
               }
               catch (Exception) { lSessionID = -1; }
            }

            if (lSessionID == -1) { return; }

            Hashtable hashUsers = new Hashtable();
            PopulateUserTable( lSessionID, cgiEngine, ref hashUsers);

            Seapine.CTableColumn[] acolumns = null;
            PopulateDefectSearchColumns(ref acolumns);

            Seapine.CRecordListSoap reclistDefects = cgiEngine.getRecordListForTable( lSessionID, "Defect", null, acolumns);

            Hashtable hashAssignedTo = new Hashtable();
            Hashtable hashDueDate    = new Hashtable();
            Hashtable hashProducts = new Hashtable();
            Hashtable hashVersions = null;
            Hashtable hashComponents = null;
            Hashtable hashData = null;
            foreach ( Seapine.CRecordRowSoap recrow in reclistDefects.records)
            {
               try
               {
                  ParseDefectRecord( lSessionID, cgiEngine, recrow, hashUsers,
                                     ref hashAssignedTo, ref hashDueDate, ref hashProducts,
                                     ref hashVersions, ref hashComponents, ref hashData);
               }
               catch (Exception) {}
            }

            msprjlib.Task objTask;
            msprjlib.Application objApplication = (msprjlib.Application) applicationObject;
            objApplication.ActiveProject.ProjectStart = "01/01/2004";
            objApplication.ActiveProject.Title = "Local";
            msprjlib.Tasks objTasks = objApplication.ActiveProject.Tasks;

            // Display the data
            foreach ( string strProduct in hashProducts.Keys)
            {
               objTask = objTasks.Add( strProduct, objTasks.Count+1);
               objTask.OutlineLevel=1;
               hashVersions = (Hashtable) hashProducts[ strProduct];

               foreach ( string strVer in hashVersions.Keys)
               {
                  objTask = objTasks.Add( strVer, objTasks.Count+1);
                  objTask.OutlineLevel=2;
                  hashComponents = (Hashtable) hashVersions[ strVer];

                  foreach ( string strComp in hashComponents.Keys)
                  {
                     objTask = objTasks.Add( strComp, objTasks.Count+1);
                     hashData = (Hashtable) hashComponents[strComp];
                     objTask.OutlineLevel=3;

                     for ( int iCntr=0; iCntr<hashData.Count; iCntr++)
                     {
                        Seapine.CDefect defect = (Seapine.CDefect)hashData[iCntr];
                        string strSummary = Convert.ToString(defect.defectnumber) +"::"+defect.summary;
                        objTask = objTasks.Add(strSummary, objTasks.Count+1);
                        objTask.ActualStart = defect.dateentered;

                        string strValue = (string)hashDueDate[ defect.defectnumber];
                        try
                        {
                           objTask.ActualFinish = strValue;
                        }
                        catch (Exception ex)
                        {
                           objTask.ActualFinish = DateTime.Now;
                        }

                        strValue = (string)hashAssignedTo[ defect.defectnumber];
                        try
                        {
                           objTask.ResourceNames = strValue.Replace(',',' ' );
                           objTask.EnterpriseNumber1 = defect.defectnumber;
                        }
                        catch (Exception ex)
                        {
                           //objTask.ResourceNames = "Not Assigned";
                        }
                        objTask.OutlineLevel=4;
                     }
                     objTask = objTasks[strComp];
                  }

                  objTask = objTasks[strVer];
                  //objTask.OutlineOutdent();
               }
            }

            cgiEngine.DatabaseLogoff( lSessionID);

	   		try
		   	{
			   	msprjlib._EProjectApp2_ProjectBeforeTaskChangeEventHandler m_evtChanged;
				   m_evtChanged = new msprjlib._EProjectApp2_ProjectBeforeTaskChangeEventHandler( SomethingChanged);
   				objApplication.ProjectBeforeTaskChange += m_evtChanged;
            }
            catch (Exception ex)
            {
            }
         }
         catch (Exception)
         {
         }
      }

      protected void PopulateDefectSearchColumns(ref Seapine.CTableColumn[] p_acolumns)
      {
         long lSize = 3;
         p_acolumns = new Seapine.CTableColumn[lSize];

         long lIter = 0;
         for (; lIter < lSize; ++lIter)
            p_acolumns[lIter] = new Seapine.CTableColumn();

         // If you change the ordering of any existing elements, then you'll have to make
         // the appropriate changes elsewhere or stuff won't work as intended.
         lIter = 0;
         p_acolumns[lIter++].name = "Number";
         p_acolumns[lIter++].name = "Currently Assigned To";
         p_acolumns[lIter++].name = "Version Found";
      }

      protected void ParseDefectRecord( long p_lSessionID,
                                        Seapine.ttsoapcgi p_cgiEngine,
                                        Seapine.CRecordRowSoap p_recrow,
                                        Hashtable p_hashUsers,
                                        ref Hashtable p_hashAssignedTo,
                                        ref Hashtable p_hashDueDate,
                                        ref Hashtable p_hashProducts,
                                        ref Hashtable p_hashVersions,
                                        ref Hashtable p_hashComponents,
                                        ref Hashtable p_hashData)
      {
         long lDNum = Convert.ToInt32( p_recrow.row[0].value);
         Seapine.CDefect defect = p_cgiEngine.getDefect( p_lSessionID, lDNum, null, false);

         if (defect != null)
         {
            // Let's convert the Assigned To name into a more display-friendly name.
            string strDisplayName = null;
            if (p_recrow.row[1].value != null)
            {
               strDisplayName = (string)p_hashUsers[(string)p_recrow.row[1].value];
               if ( strDisplayName == null)
               {  // This is assigned to multiple users!
                  int iFindPos = p_recrow.row[1].value.IndexOf(";");
                  if (iFindPos == -1) // No last name given for TTP User
                     strDisplayName = p_recrow.row[1].value;
                  else if (iFindPos >= 0)
                     strDisplayName = (string)p_hashUsers[(string)p_recrow.row[1].value.Substring(0, iFindPos)];
               }
            }

            p_hashAssignedTo.Add( lDNum, strDisplayName);

            string strProduct = defect.product;
            if ( strProduct == null) { strProduct = "N/A"; }

            string strSummary = Convert.ToString(lDNum) + "::" + defect.summary;
            if ( strSummary == null) { strSummary = "N/A"; }

            string strVersion = p_recrow.row[2].value;
            if ( strVersion == null) { strVersion = "N/A"; }

            string strComponent = defect.component;
            if ( strComponent == null) { strComponent = "N/A"; }

            // Add Version/Get
            if (!p_hashProducts.ContainsKey( strProduct))
            {
               p_hashVersions = new Hashtable();
               p_hashProducts.Add( strProduct, new Hashtable());
            }
            p_hashVersions = (Hashtable)p_hashProducts[ strProduct];

            if ( !p_hashVersions.ContainsKey( strVersion))
            {
               p_hashComponents = new Hashtable();
               p_hashVersions.Add( strVersion, p_hashComponents);
            }
            p_hashComponents = (Hashtable) p_hashVersions[ strVersion];

            if ( !p_hashComponents.ContainsKey( strComponent))
            {
               p_hashData = new Hashtable();
               p_hashComponents.Add( strComponent, p_hashData);
            }
            p_hashData = (Hashtable)p_hashComponents[ strComponent];
            p_hashData.Add( p_hashData.Count, defect);
         }
      }

      protected bool PopulateUserTable( long p_lSessionID,
                                        Seapine.ttsoapcgi p_cgiEngine,
                                        ref Hashtable p_hashUsers)
      {
         try
         {
            Seapine.CTableColumn[] acolumns = new Seapine.CTableColumn[4];
            acolumns[0] = new Seapine.CTableColumn();
            acolumns[0].name = "Name";
            acolumns[1] = new Seapine.CTableColumn();
            acolumns[1].name = "Assigned";

            Seapine.CRecordListSoap reclistUsers = p_cgiEngine.getRecordListForTable( p_lSessionID, "User", null, acolumns);
            foreach ( Seapine.CRecordRowSoap recrowUser in reclistUsers.records)
            {
               // If user has no assignments, then we won't be using their name in Project.
               if (Convert.ToInt32(recrowUser.row[1].value) == 0) { continue; }

               string strFName = "", strLName = "";

               int iFindPos = recrowUser.row[0].value.IndexOf(",");

               if (iFindPos == -1) // No last name given for TTP User
                  strFName = recrowUser.row[0].value;
               else if (iFindPos >= 0)
               {
                  strLName = recrowUser.row[0].value.Substring(0, iFindPos);
                  strFName = recrowUser.row[0].value.Substring(iFindPos + 2, recrowUser.row[0].value.Length - (iFindPos + 2));
               }

               Seapine.CUser user = p_cgiEngine.getUser( p_lSessionID, strFName, strLName);
               if (user != null)
                  p_hashUsers.Add( (string)recrowUser.row[0].value, user.firstname + " " + user.lastname);
            }
         }
         catch (Exception)
         {
         }
         return (true);
      }

   }
}

